import datetime
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.job_status import JobStatus
from ..models.task_type import TaskType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.customized_agentic_workflow_success_gpt_scorer import CustomizedAgenticWorkflowSuccessGPTScorer
    from ..models.customized_chunk_attribution_utilization_gpt_scorer import (
        CustomizedChunkAttributionUtilizationGPTScorer,
    )
    from ..models.customized_completeness_gpt_scorer import CustomizedCompletenessGPTScorer
    from ..models.customized_factuality_gpt_scorer import CustomizedFactualityGPTScorer
    from ..models.customized_ground_truth_adherence_gpt_scorer import CustomizedGroundTruthAdherenceGPTScorer
    from ..models.customized_groundedness_gpt_scorer import CustomizedGroundednessGPTScorer
    from ..models.customized_instruction_adherence_gpt_scorer import CustomizedInstructionAdherenceGPTScorer
    from ..models.customized_tool_error_rate_gpt_scorer import CustomizedToolErrorRateGPTScorer
    from ..models.customized_tool_selection_quality_gpt_scorer import CustomizedToolSelectionQualityGPTScorer
    from ..models.get_prompt_run_metrics import GetPromptRunMetrics
    from ..models.prompt_run_settings import PromptRunSettings
    from ..models.registered_scorer import RegisteredScorer
    from ..models.run_tag_db import RunTagDB
    from ..models.scorers_configuration import ScorersConfiguration


T = TypeVar("T", bound="GetPromptRun")


@_attrs_define
class GetPromptRun:
    """
    Attributes:
        created_at (datetime.datetime):
        created_by (str):
        id (str):
        job_id (str):
        job_status (JobStatus):
        last_updated_by (str):
        num_samples (int):
        updated_at (datetime.datetime):
        winner (bool):
        average_bleu (Union[None, Unset, float]):
        average_cost (Union[None, Unset, float]):
        average_hallucination (Union[None, Unset, float]):
        average_latency (Union[None, Unset, float]):
        average_rouge (Union[None, Unset, float]):
        dataset_hash (Union[None, Unset, str]):
        dataset_id (Union[None, Unset, str]):
        hallucination_severity (Union[Unset, int]):  Default: 0.
        metrics (Union[Unset, GetPromptRunMetrics]):
        model_alias (Union[None, Unset, str]):
        name (Union[None, Unset, str]):
        project_id (Union[None, Unset, str]):
        prompt_customized_scorers_configuration (Union[None, Unset,
            list[Union['CustomizedAgenticWorkflowSuccessGPTScorer', 'CustomizedChunkAttributionUtilizationGPTScorer',
            'CustomizedCompletenessGPTScorer', 'CustomizedFactualityGPTScorer', 'CustomizedGroundTruthAdherenceGPTScorer',
            'CustomizedGroundednessGPTScorer', 'CustomizedInstructionAdherenceGPTScorer',
            'CustomizedToolErrorRateGPTScorer', 'CustomizedToolSelectionQualityGPTScorer']]]):
        prompt_generated_scorers_configuration (Union[None, Unset, list[str]]):
        prompt_registered_scorers_configuration (Union[None, Unset, list['RegisteredScorer']]):
        prompt_scorers_configuration (Union[Unset, ScorersConfiguration]): Configure which scorers to enable for a
            particular prompt run.

            The keys here are sorted by their approximate execution time to execute the scorers that we anticipate will be
            the
            fastest first, and the slowest last.
        prompt_settings (Union['PromptRunSettings', None, Unset]):
        run_tags (Union[Unset, list['RunTagDB']]):
        task_type (Union[None, TaskType, Unset]):
        template_id (Union[None, Unset, str]):
        template_version (Union[None, Unset, int]):
        template_version_id (Union[None, Unset, str]):
        template_version_selected (Union[None, Unset, bool]):
        total_cost (Union[None, Unset, float]):
        total_responses (Union[Unset, int]):  Default: 0.
    """

    created_at: datetime.datetime
    created_by: str
    id: str
    job_id: str
    job_status: JobStatus
    last_updated_by: str
    num_samples: int
    updated_at: datetime.datetime
    winner: bool
    average_bleu: Union[None, Unset, float] = UNSET
    average_cost: Union[None, Unset, float] = UNSET
    average_hallucination: Union[None, Unset, float] = UNSET
    average_latency: Union[None, Unset, float] = UNSET
    average_rouge: Union[None, Unset, float] = UNSET
    dataset_hash: Union[None, Unset, str] = UNSET
    dataset_id: Union[None, Unset, str] = UNSET
    hallucination_severity: Union[Unset, int] = 0
    metrics: Union[Unset, "GetPromptRunMetrics"] = UNSET
    model_alias: Union[None, Unset, str] = UNSET
    name: Union[None, Unset, str] = UNSET
    project_id: Union[None, Unset, str] = UNSET
    prompt_customized_scorers_configuration: Union[
        None,
        Unset,
        list[
            Union[
                "CustomizedAgenticWorkflowSuccessGPTScorer",
                "CustomizedChunkAttributionUtilizationGPTScorer",
                "CustomizedCompletenessGPTScorer",
                "CustomizedFactualityGPTScorer",
                "CustomizedGroundTruthAdherenceGPTScorer",
                "CustomizedGroundednessGPTScorer",
                "CustomizedInstructionAdherenceGPTScorer",
                "CustomizedToolErrorRateGPTScorer",
                "CustomizedToolSelectionQualityGPTScorer",
            ]
        ],
    ] = UNSET
    prompt_generated_scorers_configuration: Union[None, Unset, list[str]] = UNSET
    prompt_registered_scorers_configuration: Union[None, Unset, list["RegisteredScorer"]] = UNSET
    prompt_scorers_configuration: Union[Unset, "ScorersConfiguration"] = UNSET
    prompt_settings: Union["PromptRunSettings", None, Unset] = UNSET
    run_tags: Union[Unset, list["RunTagDB"]] = UNSET
    task_type: Union[None, TaskType, Unset] = UNSET
    template_id: Union[None, Unset, str] = UNSET
    template_version: Union[None, Unset, int] = UNSET
    template_version_id: Union[None, Unset, str] = UNSET
    template_version_selected: Union[None, Unset, bool] = UNSET
    total_cost: Union[None, Unset, float] = UNSET
    total_responses: Union[Unset, int] = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.customized_agentic_workflow_success_gpt_scorer import CustomizedAgenticWorkflowSuccessGPTScorer
        from ..models.customized_chunk_attribution_utilization_gpt_scorer import (
            CustomizedChunkAttributionUtilizationGPTScorer,
        )
        from ..models.customized_completeness_gpt_scorer import CustomizedCompletenessGPTScorer
        from ..models.customized_factuality_gpt_scorer import CustomizedFactualityGPTScorer
        from ..models.customized_ground_truth_adherence_gpt_scorer import CustomizedGroundTruthAdherenceGPTScorer
        from ..models.customized_groundedness_gpt_scorer import CustomizedGroundednessGPTScorer
        from ..models.customized_instruction_adherence_gpt_scorer import CustomizedInstructionAdherenceGPTScorer
        from ..models.customized_tool_selection_quality_gpt_scorer import CustomizedToolSelectionQualityGPTScorer
        from ..models.prompt_run_settings import PromptRunSettings

        created_at = self.created_at.isoformat()

        created_by = self.created_by

        id = self.id

        job_id = self.job_id

        job_status = self.job_status.value

        last_updated_by = self.last_updated_by

        num_samples = self.num_samples

        updated_at = self.updated_at.isoformat()

        winner = self.winner

        average_bleu: Union[None, Unset, float]
        if isinstance(self.average_bleu, Unset):
            average_bleu = UNSET
        else:
            average_bleu = self.average_bleu

        average_cost: Union[None, Unset, float]
        if isinstance(self.average_cost, Unset):
            average_cost = UNSET
        else:
            average_cost = self.average_cost

        average_hallucination: Union[None, Unset, float]
        if isinstance(self.average_hallucination, Unset):
            average_hallucination = UNSET
        else:
            average_hallucination = self.average_hallucination

        average_latency: Union[None, Unset, float]
        if isinstance(self.average_latency, Unset):
            average_latency = UNSET
        else:
            average_latency = self.average_latency

        average_rouge: Union[None, Unset, float]
        if isinstance(self.average_rouge, Unset):
            average_rouge = UNSET
        else:
            average_rouge = self.average_rouge

        dataset_hash: Union[None, Unset, str]
        if isinstance(self.dataset_hash, Unset):
            dataset_hash = UNSET
        else:
            dataset_hash = self.dataset_hash

        dataset_id: Union[None, Unset, str]
        if isinstance(self.dataset_id, Unset):
            dataset_id = UNSET
        else:
            dataset_id = self.dataset_id

        hallucination_severity = self.hallucination_severity

        metrics: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.metrics, Unset):
            metrics = self.metrics.to_dict()

        model_alias: Union[None, Unset, str]
        if isinstance(self.model_alias, Unset):
            model_alias = UNSET
        else:
            model_alias = self.model_alias

        name: Union[None, Unset, str]
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        project_id: Union[None, Unset, str]
        if isinstance(self.project_id, Unset):
            project_id = UNSET
        else:
            project_id = self.project_id

        prompt_customized_scorers_configuration: Union[None, Unset, list[dict[str, Any]]]
        if isinstance(self.prompt_customized_scorers_configuration, Unset):
            prompt_customized_scorers_configuration = UNSET
        elif isinstance(self.prompt_customized_scorers_configuration, list):
            prompt_customized_scorers_configuration = []
            for (
                prompt_customized_scorers_configuration_type_0_item_data
            ) in self.prompt_customized_scorers_configuration:
                prompt_customized_scorers_configuration_type_0_item: dict[str, Any]
                if isinstance(
                    prompt_customized_scorers_configuration_type_0_item_data, CustomizedAgenticWorkflowSuccessGPTScorer
                ):
                    prompt_customized_scorers_configuration_type_0_item = (
                        prompt_customized_scorers_configuration_type_0_item_data.to_dict()
                    )
                elif isinstance(
                    prompt_customized_scorers_configuration_type_0_item_data,
                    CustomizedChunkAttributionUtilizationGPTScorer,
                ):
                    prompt_customized_scorers_configuration_type_0_item = (
                        prompt_customized_scorers_configuration_type_0_item_data.to_dict()
                    )
                elif isinstance(
                    prompt_customized_scorers_configuration_type_0_item_data, CustomizedCompletenessGPTScorer
                ):
                    prompt_customized_scorers_configuration_type_0_item = (
                        prompt_customized_scorers_configuration_type_0_item_data.to_dict()
                    )
                elif isinstance(
                    prompt_customized_scorers_configuration_type_0_item_data, CustomizedFactualityGPTScorer
                ):
                    prompt_customized_scorers_configuration_type_0_item = (
                        prompt_customized_scorers_configuration_type_0_item_data.to_dict()
                    )
                elif isinstance(
                    prompt_customized_scorers_configuration_type_0_item_data, CustomizedGroundednessGPTScorer
                ):
                    prompt_customized_scorers_configuration_type_0_item = (
                        prompt_customized_scorers_configuration_type_0_item_data.to_dict()
                    )
                elif isinstance(
                    prompt_customized_scorers_configuration_type_0_item_data, CustomizedInstructionAdherenceGPTScorer
                ):
                    prompt_customized_scorers_configuration_type_0_item = (
                        prompt_customized_scorers_configuration_type_0_item_data.to_dict()
                    )
                elif isinstance(
                    prompt_customized_scorers_configuration_type_0_item_data, CustomizedGroundTruthAdherenceGPTScorer
                ):
                    prompt_customized_scorers_configuration_type_0_item = (
                        prompt_customized_scorers_configuration_type_0_item_data.to_dict()
                    )
                elif isinstance(
                    prompt_customized_scorers_configuration_type_0_item_data, CustomizedToolSelectionQualityGPTScorer
                ):
                    prompt_customized_scorers_configuration_type_0_item = (
                        prompt_customized_scorers_configuration_type_0_item_data.to_dict()
                    )
                else:
                    prompt_customized_scorers_configuration_type_0_item = (
                        prompt_customized_scorers_configuration_type_0_item_data.to_dict()
                    )

                prompt_customized_scorers_configuration.append(prompt_customized_scorers_configuration_type_0_item)

        else:
            prompt_customized_scorers_configuration = self.prompt_customized_scorers_configuration

        prompt_generated_scorers_configuration: Union[None, Unset, list[str]]
        if isinstance(self.prompt_generated_scorers_configuration, Unset):
            prompt_generated_scorers_configuration = UNSET
        elif isinstance(self.prompt_generated_scorers_configuration, list):
            prompt_generated_scorers_configuration = self.prompt_generated_scorers_configuration

        else:
            prompt_generated_scorers_configuration = self.prompt_generated_scorers_configuration

        prompt_registered_scorers_configuration: Union[None, Unset, list[dict[str, Any]]]
        if isinstance(self.prompt_registered_scorers_configuration, Unset):
            prompt_registered_scorers_configuration = UNSET
        elif isinstance(self.prompt_registered_scorers_configuration, list):
            prompt_registered_scorers_configuration = []
            for (
                prompt_registered_scorers_configuration_type_0_item_data
            ) in self.prompt_registered_scorers_configuration:
                prompt_registered_scorers_configuration_type_0_item = (
                    prompt_registered_scorers_configuration_type_0_item_data.to_dict()
                )
                prompt_registered_scorers_configuration.append(prompt_registered_scorers_configuration_type_0_item)

        else:
            prompt_registered_scorers_configuration = self.prompt_registered_scorers_configuration

        prompt_scorers_configuration: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.prompt_scorers_configuration, Unset):
            prompt_scorers_configuration = self.prompt_scorers_configuration.to_dict()

        prompt_settings: Union[None, Unset, dict[str, Any]]
        if isinstance(self.prompt_settings, Unset):
            prompt_settings = UNSET
        elif isinstance(self.prompt_settings, PromptRunSettings):
            prompt_settings = self.prompt_settings.to_dict()
        else:
            prompt_settings = self.prompt_settings

        run_tags: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.run_tags, Unset):
            run_tags = []
            for run_tags_item_data in self.run_tags:
                run_tags_item = run_tags_item_data.to_dict()
                run_tags.append(run_tags_item)

        task_type: Union[None, Unset, int]
        if isinstance(self.task_type, Unset):
            task_type = UNSET
        elif isinstance(self.task_type, TaskType):
            task_type = self.task_type.value
        else:
            task_type = self.task_type

        template_id: Union[None, Unset, str]
        if isinstance(self.template_id, Unset):
            template_id = UNSET
        else:
            template_id = self.template_id

        template_version: Union[None, Unset, int]
        if isinstance(self.template_version, Unset):
            template_version = UNSET
        else:
            template_version = self.template_version

        template_version_id: Union[None, Unset, str]
        if isinstance(self.template_version_id, Unset):
            template_version_id = UNSET
        else:
            template_version_id = self.template_version_id

        template_version_selected: Union[None, Unset, bool]
        if isinstance(self.template_version_selected, Unset):
            template_version_selected = UNSET
        else:
            template_version_selected = self.template_version_selected

        total_cost: Union[None, Unset, float]
        if isinstance(self.total_cost, Unset):
            total_cost = UNSET
        else:
            total_cost = self.total_cost

        total_responses = self.total_responses

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "created_at": created_at,
                "created_by": created_by,
                "id": id,
                "job_id": job_id,
                "job_status": job_status,
                "last_updated_by": last_updated_by,
                "num_samples": num_samples,
                "updated_at": updated_at,
                "winner": winner,
            }
        )
        if average_bleu is not UNSET:
            field_dict["average_bleu"] = average_bleu
        if average_cost is not UNSET:
            field_dict["average_cost"] = average_cost
        if average_hallucination is not UNSET:
            field_dict["average_hallucination"] = average_hallucination
        if average_latency is not UNSET:
            field_dict["average_latency"] = average_latency
        if average_rouge is not UNSET:
            field_dict["average_rouge"] = average_rouge
        if dataset_hash is not UNSET:
            field_dict["dataset_hash"] = dataset_hash
        if dataset_id is not UNSET:
            field_dict["dataset_id"] = dataset_id
        if hallucination_severity is not UNSET:
            field_dict["hallucination_severity"] = hallucination_severity
        if metrics is not UNSET:
            field_dict["metrics"] = metrics
        if model_alias is not UNSET:
            field_dict["model_alias"] = model_alias
        if name is not UNSET:
            field_dict["name"] = name
        if project_id is not UNSET:
            field_dict["project_id"] = project_id
        if prompt_customized_scorers_configuration is not UNSET:
            field_dict["prompt_customized_scorers_configuration"] = prompt_customized_scorers_configuration
        if prompt_generated_scorers_configuration is not UNSET:
            field_dict["prompt_generated_scorers_configuration"] = prompt_generated_scorers_configuration
        if prompt_registered_scorers_configuration is not UNSET:
            field_dict["prompt_registered_scorers_configuration"] = prompt_registered_scorers_configuration
        if prompt_scorers_configuration is not UNSET:
            field_dict["prompt_scorers_configuration"] = prompt_scorers_configuration
        if prompt_settings is not UNSET:
            field_dict["prompt_settings"] = prompt_settings
        if run_tags is not UNSET:
            field_dict["run_tags"] = run_tags
        if task_type is not UNSET:
            field_dict["task_type"] = task_type
        if template_id is not UNSET:
            field_dict["template_id"] = template_id
        if template_version is not UNSET:
            field_dict["template_version"] = template_version
        if template_version_id is not UNSET:
            field_dict["template_version_id"] = template_version_id
        if template_version_selected is not UNSET:
            field_dict["template_version_selected"] = template_version_selected
        if total_cost is not UNSET:
            field_dict["total_cost"] = total_cost
        if total_responses is not UNSET:
            field_dict["total_responses"] = total_responses

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.customized_agentic_workflow_success_gpt_scorer import CustomizedAgenticWorkflowSuccessGPTScorer
        from ..models.customized_chunk_attribution_utilization_gpt_scorer import (
            CustomizedChunkAttributionUtilizationGPTScorer,
        )
        from ..models.customized_completeness_gpt_scorer import CustomizedCompletenessGPTScorer
        from ..models.customized_factuality_gpt_scorer import CustomizedFactualityGPTScorer
        from ..models.customized_ground_truth_adherence_gpt_scorer import CustomizedGroundTruthAdherenceGPTScorer
        from ..models.customized_groundedness_gpt_scorer import CustomizedGroundednessGPTScorer
        from ..models.customized_instruction_adherence_gpt_scorer import CustomizedInstructionAdherenceGPTScorer
        from ..models.customized_tool_error_rate_gpt_scorer import CustomizedToolErrorRateGPTScorer
        from ..models.customized_tool_selection_quality_gpt_scorer import CustomizedToolSelectionQualityGPTScorer
        from ..models.get_prompt_run_metrics import GetPromptRunMetrics
        from ..models.prompt_run_settings import PromptRunSettings
        from ..models.registered_scorer import RegisteredScorer
        from ..models.run_tag_db import RunTagDB
        from ..models.scorers_configuration import ScorersConfiguration

        d = src_dict.copy()
        created_at = isoparse(d.pop("created_at"))

        created_by = d.pop("created_by")

        id = d.pop("id")

        job_id = d.pop("job_id")

        job_status = JobStatus(d.pop("job_status"))

        last_updated_by = d.pop("last_updated_by")

        num_samples = d.pop("num_samples")

        updated_at = isoparse(d.pop("updated_at"))

        winner = d.pop("winner")

        def _parse_average_bleu(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_bleu = _parse_average_bleu(d.pop("average_bleu", UNSET))

        def _parse_average_cost(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_cost = _parse_average_cost(d.pop("average_cost", UNSET))

        def _parse_average_hallucination(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_hallucination = _parse_average_hallucination(d.pop("average_hallucination", UNSET))

        def _parse_average_latency(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_latency = _parse_average_latency(d.pop("average_latency", UNSET))

        def _parse_average_rouge(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_rouge = _parse_average_rouge(d.pop("average_rouge", UNSET))

        def _parse_dataset_hash(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        dataset_hash = _parse_dataset_hash(d.pop("dataset_hash", UNSET))

        def _parse_dataset_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        dataset_id = _parse_dataset_id(d.pop("dataset_id", UNSET))

        hallucination_severity = d.pop("hallucination_severity", UNSET)

        _metrics = d.pop("metrics", UNSET)
        metrics: Union[Unset, GetPromptRunMetrics]
        if isinstance(_metrics, Unset):
            metrics = UNSET
        else:
            metrics = GetPromptRunMetrics.from_dict(_metrics)

        def _parse_model_alias(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        model_alias = _parse_model_alias(d.pop("model_alias", UNSET))

        def _parse_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_project_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        project_id = _parse_project_id(d.pop("project_id", UNSET))

        def _parse_prompt_customized_scorers_configuration(
            data: object,
        ) -> Union[
            None,
            Unset,
            list[
                Union[
                    "CustomizedAgenticWorkflowSuccessGPTScorer",
                    "CustomizedChunkAttributionUtilizationGPTScorer",
                    "CustomizedCompletenessGPTScorer",
                    "CustomizedFactualityGPTScorer",
                    "CustomizedGroundTruthAdherenceGPTScorer",
                    "CustomizedGroundednessGPTScorer",
                    "CustomizedInstructionAdherenceGPTScorer",
                    "CustomizedToolErrorRateGPTScorer",
                    "CustomizedToolSelectionQualityGPTScorer",
                ]
            ],
        ]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                prompt_customized_scorers_configuration_type_0 = []
                _prompt_customized_scorers_configuration_type_0 = data
                for (
                    prompt_customized_scorers_configuration_type_0_item_data
                ) in _prompt_customized_scorers_configuration_type_0:

                    def _parse_prompt_customized_scorers_configuration_type_0_item(
                        data: object,
                    ) -> Union[
                        "CustomizedAgenticWorkflowSuccessGPTScorer",
                        "CustomizedChunkAttributionUtilizationGPTScorer",
                        "CustomizedCompletenessGPTScorer",
                        "CustomizedFactualityGPTScorer",
                        "CustomizedGroundTruthAdherenceGPTScorer",
                        "CustomizedGroundednessGPTScorer",
                        "CustomizedInstructionAdherenceGPTScorer",
                        "CustomizedToolErrorRateGPTScorer",
                        "CustomizedToolSelectionQualityGPTScorer",
                    ]:
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            prompt_customized_scorers_configuration_type_0_item_type_0 = (
                                CustomizedAgenticWorkflowSuccessGPTScorer.from_dict(data)
                            )

                            return prompt_customized_scorers_configuration_type_0_item_type_0
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            prompt_customized_scorers_configuration_type_0_item_type_1 = (
                                CustomizedChunkAttributionUtilizationGPTScorer.from_dict(data)
                            )

                            return prompt_customized_scorers_configuration_type_0_item_type_1
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            prompt_customized_scorers_configuration_type_0_item_type_2 = (
                                CustomizedCompletenessGPTScorer.from_dict(data)
                            )

                            return prompt_customized_scorers_configuration_type_0_item_type_2
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            prompt_customized_scorers_configuration_type_0_item_type_3 = (
                                CustomizedFactualityGPTScorer.from_dict(data)
                            )

                            return prompt_customized_scorers_configuration_type_0_item_type_3
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            prompt_customized_scorers_configuration_type_0_item_type_4 = (
                                CustomizedGroundednessGPTScorer.from_dict(data)
                            )

                            return prompt_customized_scorers_configuration_type_0_item_type_4
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            prompt_customized_scorers_configuration_type_0_item_type_5 = (
                                CustomizedInstructionAdherenceGPTScorer.from_dict(data)
                            )

                            return prompt_customized_scorers_configuration_type_0_item_type_5
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            prompt_customized_scorers_configuration_type_0_item_type_6 = (
                                CustomizedGroundTruthAdherenceGPTScorer.from_dict(data)
                            )

                            return prompt_customized_scorers_configuration_type_0_item_type_6
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            prompt_customized_scorers_configuration_type_0_item_type_7 = (
                                CustomizedToolSelectionQualityGPTScorer.from_dict(data)
                            )

                            return prompt_customized_scorers_configuration_type_0_item_type_7
                        except:  # noqa: E722
                            pass
                        if not isinstance(data, dict):
                            raise TypeError()
                        prompt_customized_scorers_configuration_type_0_item_type_8 = (
                            CustomizedToolErrorRateGPTScorer.from_dict(data)
                        )

                        return prompt_customized_scorers_configuration_type_0_item_type_8

                    prompt_customized_scorers_configuration_type_0_item = (
                        _parse_prompt_customized_scorers_configuration_type_0_item(
                            prompt_customized_scorers_configuration_type_0_item_data
                        )
                    )

                    prompt_customized_scorers_configuration_type_0.append(
                        prompt_customized_scorers_configuration_type_0_item
                    )

                return prompt_customized_scorers_configuration_type_0
            except:  # noqa: E722
                pass
            return cast(
                Union[
                    None,
                    Unset,
                    list[
                        Union[
                            "CustomizedAgenticWorkflowSuccessGPTScorer",
                            "CustomizedChunkAttributionUtilizationGPTScorer",
                            "CustomizedCompletenessGPTScorer",
                            "CustomizedFactualityGPTScorer",
                            "CustomizedGroundTruthAdherenceGPTScorer",
                            "CustomizedGroundednessGPTScorer",
                            "CustomizedInstructionAdherenceGPTScorer",
                            "CustomizedToolErrorRateGPTScorer",
                            "CustomizedToolSelectionQualityGPTScorer",
                        ]
                    ],
                ],
                data,
            )

        prompt_customized_scorers_configuration = _parse_prompt_customized_scorers_configuration(
            d.pop("prompt_customized_scorers_configuration", UNSET)
        )

        def _parse_prompt_generated_scorers_configuration(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                prompt_generated_scorers_configuration_type_0 = cast(list[str], data)

                return prompt_generated_scorers_configuration_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        prompt_generated_scorers_configuration = _parse_prompt_generated_scorers_configuration(
            d.pop("prompt_generated_scorers_configuration", UNSET)
        )

        def _parse_prompt_registered_scorers_configuration(
            data: object,
        ) -> Union[None, Unset, list["RegisteredScorer"]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                prompt_registered_scorers_configuration_type_0 = []
                _prompt_registered_scorers_configuration_type_0 = data
                for (
                    prompt_registered_scorers_configuration_type_0_item_data
                ) in _prompt_registered_scorers_configuration_type_0:
                    prompt_registered_scorers_configuration_type_0_item = RegisteredScorer.from_dict(
                        prompt_registered_scorers_configuration_type_0_item_data
                    )

                    prompt_registered_scorers_configuration_type_0.append(
                        prompt_registered_scorers_configuration_type_0_item
                    )

                return prompt_registered_scorers_configuration_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list["RegisteredScorer"]], data)

        prompt_registered_scorers_configuration = _parse_prompt_registered_scorers_configuration(
            d.pop("prompt_registered_scorers_configuration", UNSET)
        )

        _prompt_scorers_configuration = d.pop("prompt_scorers_configuration", UNSET)
        prompt_scorers_configuration: Union[Unset, ScorersConfiguration]
        if isinstance(_prompt_scorers_configuration, Unset):
            prompt_scorers_configuration = UNSET
        else:
            prompt_scorers_configuration = ScorersConfiguration.from_dict(_prompt_scorers_configuration)

        def _parse_prompt_settings(data: object) -> Union["PromptRunSettings", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                prompt_settings_type_0 = PromptRunSettings.from_dict(data)

                return prompt_settings_type_0
            except:  # noqa: E722
                pass
            return cast(Union["PromptRunSettings", None, Unset], data)

        prompt_settings = _parse_prompt_settings(d.pop("prompt_settings", UNSET))

        run_tags = []
        _run_tags = d.pop("run_tags", UNSET)
        for run_tags_item_data in _run_tags or []:
            run_tags_item = RunTagDB.from_dict(run_tags_item_data)

            run_tags.append(run_tags_item)

        def _parse_task_type(data: object) -> Union[None, TaskType, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, int):
                    raise TypeError()
                task_type_type_0 = TaskType(data)

                return task_type_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, TaskType, Unset], data)

        task_type = _parse_task_type(d.pop("task_type", UNSET))

        def _parse_template_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        template_id = _parse_template_id(d.pop("template_id", UNSET))

        def _parse_template_version(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        template_version = _parse_template_version(d.pop("template_version", UNSET))

        def _parse_template_version_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        template_version_id = _parse_template_version_id(d.pop("template_version_id", UNSET))

        def _parse_template_version_selected(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        template_version_selected = _parse_template_version_selected(d.pop("template_version_selected", UNSET))

        def _parse_total_cost(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        total_cost = _parse_total_cost(d.pop("total_cost", UNSET))

        total_responses = d.pop("total_responses", UNSET)

        get_prompt_run = cls(
            created_at=created_at,
            created_by=created_by,
            id=id,
            job_id=job_id,
            job_status=job_status,
            last_updated_by=last_updated_by,
            num_samples=num_samples,
            updated_at=updated_at,
            winner=winner,
            average_bleu=average_bleu,
            average_cost=average_cost,
            average_hallucination=average_hallucination,
            average_latency=average_latency,
            average_rouge=average_rouge,
            dataset_hash=dataset_hash,
            dataset_id=dataset_id,
            hallucination_severity=hallucination_severity,
            metrics=metrics,
            model_alias=model_alias,
            name=name,
            project_id=project_id,
            prompt_customized_scorers_configuration=prompt_customized_scorers_configuration,
            prompt_generated_scorers_configuration=prompt_generated_scorers_configuration,
            prompt_registered_scorers_configuration=prompt_registered_scorers_configuration,
            prompt_scorers_configuration=prompt_scorers_configuration,
            prompt_settings=prompt_settings,
            run_tags=run_tags,
            task_type=task_type,
            template_id=template_id,
            template_version=template_version,
            template_version_id=template_version_id,
            template_version_selected=template_version_selected,
            total_cost=total_cost,
            total_responses=total_responses,
        )

        get_prompt_run.additional_properties = d
        return get_prompt_run

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.feedback_rating_operation_type import FeedbackRatingOperationType

T = TypeVar("T", bound="BulkFeedbackRatingStatus")


@_attrs_define
class BulkFeedbackRatingStatus:
    """
    Attributes:
        index (int):
        message (Union[None, str]):
        operation (FeedbackRatingOperationType): Operation type for bulk feedback rating operations.
        success (bool):
    """

    index: int
    message: Union[None, str]
    operation: FeedbackRatingOperationType
    success: bool
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        index = self.index

        message: Union[None, str]
        message = self.message

        operation = self.operation.value

        success = self.success

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"index": index, "message": message, "operation": operation, "success": success})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        index = d.pop("index")

        def _parse_message(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        message = _parse_message(d.pop("message"))

        operation = FeedbackRatingOperationType(d.pop("operation"))

        success = d.pop("success")

        bulk_feedback_rating_status = cls(index=index, message=message, operation=operation, success=success)

        bulk_feedback_rating_status.additional_properties = d
        return bulk_feedback_rating_status

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.split import Split
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.filter_params import FilterParams


T = TypeVar("T", bound="FilterPercentageResponse")


@_attrs_define
class FilterPercentageResponse:
    """
    Attributes:
        percentage (float):
        sample_count (int):
        compare_to (Union[None, Split, Unset]):
        feature_noise_pct (Union[None, Unset, float]):
        filter_params (Union[Unset, FilterParams]):
        gold_box_count (Union[None, Unset, int]):
        gold_box_percentage (Union[None, Unset, float]):
        gold_polygon_count (Union[None, Unset, int]):
        gold_polygon_percentage (Union[None, Unset, float]):
        label_noise_pct (Union[None, Unset, float]):
        map_threshold (Union[Unset, float]):  Default: 0.5.
        meta_cols (Union[None, Unset, list[str]]):
        pred_box_count (Union[None, Unset, int]):
        pred_box_percentage (Union[None, Unset, float]):
        pred_polygon_count (Union[None, Unset, int]):
        pred_polygon_percentage (Union[None, Unset, float]):
        sample_drifted_percentage (Union[None, Unset, float]):
        sample_easy_percentage (Union[None, Unset, float]):
        sample_error_percentage (Union[None, Unset, float]):
        sample_hard_percentage (Union[None, Unset, float]):
        sample_misclassified_percentage (Union[None, Unset, float]):
        sample_mislabeled_percentage (Union[None, Unset, float]):
        sample_otb_percentage (Union[None, Unset, float]):
        span_count (Union[None, Unset, int]):
        span_percentage (Union[None, Unset, float]):
        task (Union[None, Unset, str]):
    """

    percentage: float
    sample_count: int
    compare_to: Union[None, Split, Unset] = UNSET
    feature_noise_pct: Union[None, Unset, float] = UNSET
    filter_params: Union[Unset, "FilterParams"] = UNSET
    gold_box_count: Union[None, Unset, int] = UNSET
    gold_box_percentage: Union[None, Unset, float] = UNSET
    gold_polygon_count: Union[None, Unset, int] = UNSET
    gold_polygon_percentage: Union[None, Unset, float] = UNSET
    label_noise_pct: Union[None, Unset, float] = UNSET
    map_threshold: Union[Unset, float] = 0.5
    meta_cols: Union[None, Unset, list[str]] = UNSET
    pred_box_count: Union[None, Unset, int] = UNSET
    pred_box_percentage: Union[None, Unset, float] = UNSET
    pred_polygon_count: Union[None, Unset, int] = UNSET
    pred_polygon_percentage: Union[None, Unset, float] = UNSET
    sample_drifted_percentage: Union[None, Unset, float] = UNSET
    sample_easy_percentage: Union[None, Unset, float] = UNSET
    sample_error_percentage: Union[None, Unset, float] = UNSET
    sample_hard_percentage: Union[None, Unset, float] = UNSET
    sample_misclassified_percentage: Union[None, Unset, float] = UNSET
    sample_mislabeled_percentage: Union[None, Unset, float] = UNSET
    sample_otb_percentage: Union[None, Unset, float] = UNSET
    span_count: Union[None, Unset, int] = UNSET
    span_percentage: Union[None, Unset, float] = UNSET
    task: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        percentage = self.percentage

        sample_count = self.sample_count

        compare_to: Union[None, Unset, str]
        if isinstance(self.compare_to, Unset):
            compare_to = UNSET
        elif isinstance(self.compare_to, Split):
            compare_to = self.compare_to.value
        else:
            compare_to = self.compare_to

        feature_noise_pct: Union[None, Unset, float]
        if isinstance(self.feature_noise_pct, Unset):
            feature_noise_pct = UNSET
        else:
            feature_noise_pct = self.feature_noise_pct

        filter_params: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.filter_params, Unset):
            filter_params = self.filter_params.to_dict()

        gold_box_count: Union[None, Unset, int]
        if isinstance(self.gold_box_count, Unset):
            gold_box_count = UNSET
        else:
            gold_box_count = self.gold_box_count

        gold_box_percentage: Union[None, Unset, float]
        if isinstance(self.gold_box_percentage, Unset):
            gold_box_percentage = UNSET
        else:
            gold_box_percentage = self.gold_box_percentage

        gold_polygon_count: Union[None, Unset, int]
        if isinstance(self.gold_polygon_count, Unset):
            gold_polygon_count = UNSET
        else:
            gold_polygon_count = self.gold_polygon_count

        gold_polygon_percentage: Union[None, Unset, float]
        if isinstance(self.gold_polygon_percentage, Unset):
            gold_polygon_percentage = UNSET
        else:
            gold_polygon_percentage = self.gold_polygon_percentage

        label_noise_pct: Union[None, Unset, float]
        if isinstance(self.label_noise_pct, Unset):
            label_noise_pct = UNSET
        else:
            label_noise_pct = self.label_noise_pct

        map_threshold = self.map_threshold

        meta_cols: Union[None, Unset, list[str]]
        if isinstance(self.meta_cols, Unset):
            meta_cols = UNSET
        elif isinstance(self.meta_cols, list):
            meta_cols = self.meta_cols

        else:
            meta_cols = self.meta_cols

        pred_box_count: Union[None, Unset, int]
        if isinstance(self.pred_box_count, Unset):
            pred_box_count = UNSET
        else:
            pred_box_count = self.pred_box_count

        pred_box_percentage: Union[None, Unset, float]
        if isinstance(self.pred_box_percentage, Unset):
            pred_box_percentage = UNSET
        else:
            pred_box_percentage = self.pred_box_percentage

        pred_polygon_count: Union[None, Unset, int]
        if isinstance(self.pred_polygon_count, Unset):
            pred_polygon_count = UNSET
        else:
            pred_polygon_count = self.pred_polygon_count

        pred_polygon_percentage: Union[None, Unset, float]
        if isinstance(self.pred_polygon_percentage, Unset):
            pred_polygon_percentage = UNSET
        else:
            pred_polygon_percentage = self.pred_polygon_percentage

        sample_drifted_percentage: Union[None, Unset, float]
        if isinstance(self.sample_drifted_percentage, Unset):
            sample_drifted_percentage = UNSET
        else:
            sample_drifted_percentage = self.sample_drifted_percentage

        sample_easy_percentage: Union[None, Unset, float]
        if isinstance(self.sample_easy_percentage, Unset):
            sample_easy_percentage = UNSET
        else:
            sample_easy_percentage = self.sample_easy_percentage

        sample_error_percentage: Union[None, Unset, float]
        if isinstance(self.sample_error_percentage, Unset):
            sample_error_percentage = UNSET
        else:
            sample_error_percentage = self.sample_error_percentage

        sample_hard_percentage: Union[None, Unset, float]
        if isinstance(self.sample_hard_percentage, Unset):
            sample_hard_percentage = UNSET
        else:
            sample_hard_percentage = self.sample_hard_percentage

        sample_misclassified_percentage: Union[None, Unset, float]
        if isinstance(self.sample_misclassified_percentage, Unset):
            sample_misclassified_percentage = UNSET
        else:
            sample_misclassified_percentage = self.sample_misclassified_percentage

        sample_mislabeled_percentage: Union[None, Unset, float]
        if isinstance(self.sample_mislabeled_percentage, Unset):
            sample_mislabeled_percentage = UNSET
        else:
            sample_mislabeled_percentage = self.sample_mislabeled_percentage

        sample_otb_percentage: Union[None, Unset, float]
        if isinstance(self.sample_otb_percentage, Unset):
            sample_otb_percentage = UNSET
        else:
            sample_otb_percentage = self.sample_otb_percentage

        span_count: Union[None, Unset, int]
        if isinstance(self.span_count, Unset):
            span_count = UNSET
        else:
            span_count = self.span_count

        span_percentage: Union[None, Unset, float]
        if isinstance(self.span_percentage, Unset):
            span_percentage = UNSET
        else:
            span_percentage = self.span_percentage

        task: Union[None, Unset, str]
        if isinstance(self.task, Unset):
            task = UNSET
        else:
            task = self.task

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"percentage": percentage, "sample_count": sample_count})
        if compare_to is not UNSET:
            field_dict["compare_to"] = compare_to
        if feature_noise_pct is not UNSET:
            field_dict["feature_noise_pct"] = feature_noise_pct
        if filter_params is not UNSET:
            field_dict["filter_params"] = filter_params
        if gold_box_count is not UNSET:
            field_dict["gold_box_count"] = gold_box_count
        if gold_box_percentage is not UNSET:
            field_dict["gold_box_percentage"] = gold_box_percentage
        if gold_polygon_count is not UNSET:
            field_dict["gold_polygon_count"] = gold_polygon_count
        if gold_polygon_percentage is not UNSET:
            field_dict["gold_polygon_percentage"] = gold_polygon_percentage
        if label_noise_pct is not UNSET:
            field_dict["label_noise_pct"] = label_noise_pct
        if map_threshold is not UNSET:
            field_dict["map_threshold"] = map_threshold
        if meta_cols is not UNSET:
            field_dict["meta_cols"] = meta_cols
        if pred_box_count is not UNSET:
            field_dict["pred_box_count"] = pred_box_count
        if pred_box_percentage is not UNSET:
            field_dict["pred_box_percentage"] = pred_box_percentage
        if pred_polygon_count is not UNSET:
            field_dict["pred_polygon_count"] = pred_polygon_count
        if pred_polygon_percentage is not UNSET:
            field_dict["pred_polygon_percentage"] = pred_polygon_percentage
        if sample_drifted_percentage is not UNSET:
            field_dict["sample_drifted_percentage"] = sample_drifted_percentage
        if sample_easy_percentage is not UNSET:
            field_dict["sample_easy_percentage"] = sample_easy_percentage
        if sample_error_percentage is not UNSET:
            field_dict["sample_error_percentage"] = sample_error_percentage
        if sample_hard_percentage is not UNSET:
            field_dict["sample_hard_percentage"] = sample_hard_percentage
        if sample_misclassified_percentage is not UNSET:
            field_dict["sample_misclassified_percentage"] = sample_misclassified_percentage
        if sample_mislabeled_percentage is not UNSET:
            field_dict["sample_mislabeled_percentage"] = sample_mislabeled_percentage
        if sample_otb_percentage is not UNSET:
            field_dict["sample_otb_percentage"] = sample_otb_percentage
        if span_count is not UNSET:
            field_dict["span_count"] = span_count
        if span_percentage is not UNSET:
            field_dict["span_percentage"] = span_percentage
        if task is not UNSET:
            field_dict["task"] = task

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.filter_params import FilterParams

        d = src_dict.copy()
        percentage = d.pop("percentage")

        sample_count = d.pop("sample_count")

        def _parse_compare_to(data: object) -> Union[None, Split, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                compare_to_type_0 = Split(data)

                return compare_to_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Split, Unset], data)

        compare_to = _parse_compare_to(d.pop("compare_to", UNSET))

        def _parse_feature_noise_pct(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        feature_noise_pct = _parse_feature_noise_pct(d.pop("feature_noise_pct", UNSET))

        _filter_params = d.pop("filter_params", UNSET)
        filter_params: Union[Unset, FilterParams]
        if isinstance(_filter_params, Unset):
            filter_params = UNSET
        else:
            filter_params = FilterParams.from_dict(_filter_params)

        def _parse_gold_box_count(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        gold_box_count = _parse_gold_box_count(d.pop("gold_box_count", UNSET))

        def _parse_gold_box_percentage(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        gold_box_percentage = _parse_gold_box_percentage(d.pop("gold_box_percentage", UNSET))

        def _parse_gold_polygon_count(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        gold_polygon_count = _parse_gold_polygon_count(d.pop("gold_polygon_count", UNSET))

        def _parse_gold_polygon_percentage(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        gold_polygon_percentage = _parse_gold_polygon_percentage(d.pop("gold_polygon_percentage", UNSET))

        def _parse_label_noise_pct(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        label_noise_pct = _parse_label_noise_pct(d.pop("label_noise_pct", UNSET))

        map_threshold = d.pop("map_threshold", UNSET)

        def _parse_meta_cols(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                meta_cols_type_0 = cast(list[str], data)

                return meta_cols_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        meta_cols = _parse_meta_cols(d.pop("meta_cols", UNSET))

        def _parse_pred_box_count(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        pred_box_count = _parse_pred_box_count(d.pop("pred_box_count", UNSET))

        def _parse_pred_box_percentage(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        pred_box_percentage = _parse_pred_box_percentage(d.pop("pred_box_percentage", UNSET))

        def _parse_pred_polygon_count(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        pred_polygon_count = _parse_pred_polygon_count(d.pop("pred_polygon_count", UNSET))

        def _parse_pred_polygon_percentage(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        pred_polygon_percentage = _parse_pred_polygon_percentage(d.pop("pred_polygon_percentage", UNSET))

        def _parse_sample_drifted_percentage(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        sample_drifted_percentage = _parse_sample_drifted_percentage(d.pop("sample_drifted_percentage", UNSET))

        def _parse_sample_easy_percentage(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        sample_easy_percentage = _parse_sample_easy_percentage(d.pop("sample_easy_percentage", UNSET))

        def _parse_sample_error_percentage(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        sample_error_percentage = _parse_sample_error_percentage(d.pop("sample_error_percentage", UNSET))

        def _parse_sample_hard_percentage(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        sample_hard_percentage = _parse_sample_hard_percentage(d.pop("sample_hard_percentage", UNSET))

        def _parse_sample_misclassified_percentage(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        sample_misclassified_percentage = _parse_sample_misclassified_percentage(
            d.pop("sample_misclassified_percentage", UNSET)
        )

        def _parse_sample_mislabeled_percentage(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        sample_mislabeled_percentage = _parse_sample_mislabeled_percentage(d.pop("sample_mislabeled_percentage", UNSET))

        def _parse_sample_otb_percentage(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        sample_otb_percentage = _parse_sample_otb_percentage(d.pop("sample_otb_percentage", UNSET))

        def _parse_span_count(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        span_count = _parse_span_count(d.pop("span_count", UNSET))

        def _parse_span_percentage(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        span_percentage = _parse_span_percentage(d.pop("span_percentage", UNSET))

        def _parse_task(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        task = _parse_task(d.pop("task", UNSET))

        filter_percentage_response = cls(
            percentage=percentage,
            sample_count=sample_count,
            compare_to=compare_to,
            feature_noise_pct=feature_noise_pct,
            filter_params=filter_params,
            gold_box_count=gold_box_count,
            gold_box_percentage=gold_box_percentage,
            gold_polygon_count=gold_polygon_count,
            gold_polygon_percentage=gold_polygon_percentage,
            label_noise_pct=label_noise_pct,
            map_threshold=map_threshold,
            meta_cols=meta_cols,
            pred_box_count=pred_box_count,
            pred_box_percentage=pred_box_percentage,
            pred_polygon_count=pred_polygon_count,
            pred_polygon_percentage=pred_polygon_percentage,
            sample_drifted_percentage=sample_drifted_percentage,
            sample_easy_percentage=sample_easy_percentage,
            sample_error_percentage=sample_error_percentage,
            sample_hard_percentage=sample_hard_percentage,
            sample_misclassified_percentage=sample_misclassified_percentage,
            sample_mislabeled_percentage=sample_mislabeled_percentage,
            sample_otb_percentage=sample_otb_percentage,
            span_count=span_count,
            span_percentage=span_percentage,
            task=task,
        )

        filter_percentage_response.additional_properties = d
        return filter_percentage_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.put_prompt_score_request import PutPromptScoreRequest
from ...models.put_prompt_score_response import PutPromptScoreResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_id: str,
    run_id: str,
    scorer_name: str,
    *,
    body: PutPromptScoreRequest,
    monitor_batch_id: Union[None, Unset, str] = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    json_monitor_batch_id: Union[None, Unset, str]
    if isinstance(monitor_batch_id, Unset):
        json_monitor_batch_id = UNSET
    else:
        json_monitor_batch_id = monitor_batch_id
    params["monitor_batch_id"] = json_monitor_batch_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "put",
        "url": f"/projects/{project_id}/runs/{run_id}/prompts/scorers/{scorer_name}",
        "params": params,
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[HTTPValidationError, PutPromptScoreResponse]]:
    if response.status_code == 200:
        response_200 = PutPromptScoreResponse.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[HTTPValidationError, PutPromptScoreResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_id: str,
    run_id: str,
    scorer_name: str,
    *,
    client: AuthenticatedClient,
    body: PutPromptScoreRequest,
    monitor_batch_id: Union[None, Unset, str] = UNSET,
) -> Response[Union[HTTPValidationError, PutPromptScoreResponse]]:
    """Put Score

    Args:
        project_id (str):
        run_id (str):
        scorer_name (str):
        monitor_batch_id (Union[None, Unset, str]):
        body (PutPromptScoreRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, PutPromptScoreResponse]]
    """

    kwargs = _get_kwargs(
        project_id=project_id, run_id=run_id, scorer_name=scorer_name, body=body, monitor_batch_id=monitor_batch_id
    )

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    project_id: str,
    run_id: str,
    scorer_name: str,
    *,
    client: AuthenticatedClient,
    body: PutPromptScoreRequest,
    monitor_batch_id: Union[None, Unset, str] = UNSET,
) -> Optional[Union[HTTPValidationError, PutPromptScoreResponse]]:
    """Put Score

    Args:
        project_id (str):
        run_id (str):
        scorer_name (str):
        monitor_batch_id (Union[None, Unset, str]):
        body (PutPromptScoreRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, PutPromptScoreResponse]
    """

    return sync_detailed(
        project_id=project_id,
        run_id=run_id,
        scorer_name=scorer_name,
        client=client,
        body=body,
        monitor_batch_id=monitor_batch_id,
    ).parsed


async def asyncio_detailed(
    project_id: str,
    run_id: str,
    scorer_name: str,
    *,
    client: AuthenticatedClient,
    body: PutPromptScoreRequest,
    monitor_batch_id: Union[None, Unset, str] = UNSET,
) -> Response[Union[HTTPValidationError, PutPromptScoreResponse]]:
    """Put Score

    Args:
        project_id (str):
        run_id (str):
        scorer_name (str):
        monitor_batch_id (Union[None, Unset, str]):
        body (PutPromptScoreRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, PutPromptScoreResponse]]
    """

    kwargs = _get_kwargs(
        project_id=project_id, run_id=run_id, scorer_name=scorer_name, body=body, monitor_batch_id=monitor_batch_id
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: str,
    run_id: str,
    scorer_name: str,
    *,
    client: AuthenticatedClient,
    body: PutPromptScoreRequest,
    monitor_batch_id: Union[None, Unset, str] = UNSET,
) -> Optional[Union[HTTPValidationError, PutPromptScoreResponse]]:
    """Put Score

    Args:
        project_id (str):
        run_id (str):
        scorer_name (str):
        monitor_batch_id (Union[None, Unset, str]):
        body (PutPromptScoreRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, PutPromptScoreResponse]
    """

    return (
        await asyncio_detailed(
            project_id=project_id,
            run_id=run_id,
            scorer_name=scorer_name,
            client=client,
            body=body,
            monitor_batch_id=monitor_batch_id,
        )
    ).parsed
